﻿/*
 * This is free and unencumbered software released into the public domain.
 * 
 * Anyone is free to copy, modify, publish, use, compile, sell, or
 * distribute this software, either in source code form or as a compiled
 * binary, for any purpose, commercial or non-commercial, and by any
 * means.
 * 
 * In jurisdictions that recognize copyright laws, the author or authors
 * of this software dedicate any and all copyright interest in the
 * software to the public domain. We make this dedication for the benefit
 * of the public at large and to the detriment of our heirs and
 * successors. We intend this dedication to be an overt act of
 * relinquishment in perpetuity of all present and future rights to this
 * software under copyright law.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * 
 * For more information, please refer to <http://unlicense.org/>
 */

/*
 * The inspiration for this code comes from a SuperUser question (http://superuser.com/questions/549028/how-can-i-prevent-ask-com-toolbar-from-being-installed-every-time-java-is-update).
 * This tool adds a registry key that disables the Java Updater from even asking the user
 * to install the Ask.com toolbar.
 * 
 */

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Microsoft.Win32;

namespace AskToolbarInstallDisabler
{
  static class Program
  {
    /// <summary>
    /// The main entry point for the application.
    /// </summary>
    [STAThread]
    static void Main(string[] args)
    {
      if (!UacHelper.IsProcessElevated)
      {
        MessageBox.Show("This program must be run as an administrator.  To do this, right-click on the program and select 'Run As Administrator'.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        return;
      }

      string status = "DISABLE";
      if (args.Length >= 1)
      {
        if (args[0].ToLower() == "--enable")
        {
          status = "ENABLE";
        }
      }

      string regKey = @"HKEY_LOCAL_MACHINE\SOFTWARE\JavaSoft";
      Registry.SetValue(regKey, "SPONSORS", "DISABLE", RegistryValueKind.String);

      if (SixtyFour.Is64BitOperatingSystem())
      {
        string regKey2 = @"HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\JavaSoft";
        Registry.SetValue(regKey2, "SPONSORS", "DISABLE", RegistryValueKind.String);
      }

      MessageBox.Show("Ask Toolbar Installer has been " + status + "D", "Success", MessageBoxButtons.OK, MessageBoxIcon.None);
    }
  }
}
